# Songwrite
# Copyright (C) 2001-2002 Jean-Baptiste LAMY
#
# This program is free software. See README or LICENSE for the license terms.

from __future__ import nested_scopes

import Tkinter
import editobj, editobj.editor as editor, editobj.eventobj as eventobj
import song, globdef, drum, view

editobj.TRANSLATOR = _

song.Song.children = property(lambda self: self.partitions)

class LabeledList:
  def __init__(self, label, l = None):
    self._label = label
    self.items  = l
    
  def __unicode__(self): return self._label
  
def _partition_children(self):
  if self.view.__class__.__name__ in ("Tablature", "DrumView"):
    return self.view.strings
    return [LabeledList(_("Strings"), self.view.strings)]
  return []

song.Partition.items = property(_partition_children)
  
def NewString(parent):
  import tablature, drum
  if   isinstance(parent.view, tablature.Tablature): return tablature.String()
  elif isinstance(parent.view, drum.DrumView      ): return drum     .String()
  
editobj.EVAL_ENV["song"]      = song
editobj.EVAL_ENV["NewString"] = NewString
editor.register_auto_child(song.Partition, "NewString(parent)")

_NORMAL_NOTE = ("Note", "LinkedNote")


def _set_view(self, viewtype):
  canvas = self.view.canvas
  self.setviewtype(viewtype)
  canvas.draw()
  
song.TemporalData.set_view = _set_view

editor.register_attr("view", None)
editor.register_attr("view", editor.ListEditor(view.view_types), song.Partition)



def _splitter(s):
  a, b = s.split(None, 1)
  return int(a), b

INSTRUMENTS = dict(map(_splitter, globdef.translator.gettext("__instru__").split("\n")))

range_0_127 = editor.RangeEditor(0, 127)

class NoteValueEditor(editor.Editor, Tkinter.Frame):
  def __init__(self, master, obj, attr):
    editor.Editor.__init__(self, master, obj, attr)
    Tkinter.Frame.__init__(self, master)
    self.columnconfigure(0, weight = 0)
    self.columnconfigure(1, weight = 1)
    
    self.label = Tkinter.Label(self, width = 8)
    self.label.grid()
    
    self.scale = Tkinter.Scale(self, bigincrement = 12, command = self.update_label, orient = Tkinter.HORIZONTAL, from_ = 0, to = 90)
    self.scale.grid(row = 0, col = 1, sticky = "nsew")
    self.scale.bind("<ButtonRelease>", self.validate)
    self.update()
    
  def update_label(self, val):
    self.label.configure(text = song.note_label(int(val)))
    
  def validate(self, event): self.set_value(self.scale.get())
  
  def get_value(self): return int(getattr(self.obj, self.attr, 0))
  
  def update(self):
    try:
      val = getattr(self.obj, self.attr)
    except AttributeError: return 0 # Not readable => do not update !
    self.scale.set(val)
    self.label.configure(text = song.note_label(val))
    return 1
  
class DrumPatchEditor(editor.Editor, Tkinter.Frame):
  def __init__(self, master, obj, attr):
    editor.Editor.__init__(self, master, obj, attr)
    Tkinter.Frame.__init__(self, master)
    self.columnconfigure(0, weight = 0)
    self.columnconfigure(1, weight = 1)
    
    self.scale = Tkinter.Scale(self, bigincrement = 12, command = self.update_label, orient = Tkinter.HORIZONTAL, from_ = 35, to = 81)
    self.scale.grid(row = 0, col = 1, sticky = "nsew")
    self.scale.bind("<ButtonRelease>", self.validate)
    self.update()
    
  def update_label(self, val):
    self.scale.configure(label = drum.PATCHES[int(val) - 35])
    
  def validate(self, event): self.set_value(self.scale.get())
  
  def get_value(self): return int(getattr(self.obj, self.attr, 0))
  
  def update(self):
    try:
      val = getattr(self.obj, self.attr)
    except AttributeError: return 0 # Not readable => do not update !
    self.scale.set(val)
    self.scale.configure(label = drum.PATCHES[val - 35])
    return 1
  
class DurationEditor(editor.Editor, Tkinter.Frame):
  def __init__(self, master, obj, attr):
    editor.Editor.__init__(self, master, obj, attr)
    Tkinter.Frame.__init__(self, master)
    self.columnconfigure(0, weight = 1)
    self.columnconfigure(1, weight = 1)
    
    self.scale = Tkinter.Scale(self, bigincrement = 12, orient = Tkinter.HORIZONTAL, from_ = 1, to = 6)
    self.scale.bind("<ButtonRelease>", self.validate)
    self.scale.grid(row = 0, col = 0, columnspan = 2, sticky = "nsew")
    
    self.doted = Tkinter.BooleanVar()
    Tkinter.Checkbutton(self, text = _("Doted"), variable = self.doted, command = self.validate).grid(row = 1, col = 0)
    
    self.triplet = Tkinter.BooleanVar()
    Tkinter.Checkbutton(self, text = _("Triplet"), variable = self.triplet, command = self.validate).grid(row = 1, col = 1)
    
    self.update()
    
  def validate(self, event = None):
    val = 6 * 2 ** self.scale.get()
    if   self.doted  .get(): val = int(val * 1.5)
    elif self.triplet.get(): val = int(val / 1.5)
    self.set_value(val)
    self.scale.configure(label = song.duration_label(val))
    
  def get_value(self): return int(getattr(self.obj, self.attr, 0))
  
  def update(self):
    try:
      val = getattr(self.obj, self.attr)
    except AttributeError: return 0 # Not readable => do not update !
    
    self.scale.configure(label = song.duration_label(val))
    
    if val in (9, 18, 36, 72, 144, 288, 576):
      val = int(val / 1.5)
      self.doted.set(1)
    else:
      self.doted.set(0)
      
    if val in (4,  8, 16, 32,  64, 128, 256):
      val = int(val * 1.5)
      self.triplet.set(1)
    else:
      self.triplet.set(0)
      
    self.scale.set({ 0: 0, 6 : 0, 12 : 1, 24: 2, 48 : 3, 96 : 4, 192 : 5, 384 : 6 }[val])
    return 1
  



editor.register_attr("partitions"         , None)
editor.register_attr("mesures"            , None)
editor.register_attr("version"            , None)
editor.register_attr("title"              , editor.StringEditor)
editor.register_attr("authors"            , editor.StringEditor)
editor.register_attr("copyright"          , editor.StringEditor)
editor.register_attr("comments"           , editor.TextEditor)
editor.register_attr("lang"               , editor.ListEditor((u"fr", u"en", u"de", u"es", u"it", u"pt", u"sw")))

editor.register_attr("song"               , None)
editor.register_attr("notes"              , None)
editor.register_attr("lyrics"             , None)
editor.register_attr("oldview"            , None)
editor.register_attr("header"             , editor.TextEditor)
editor.register_attr("chorus"             , range_0_127)
editor.register_attr("reverb"             , range_0_127)
editor.register_attr("muted"              , editor.BoolEditor)
editor.register_attr("volume"             , editor.RangeEditor(0, 255))
editor.register_attr("instrument"         , editor.EnumEditor(INSTRUMENTS))
editor.register_attr("capo"               , editor.IntEditor)

def _set_capo(self, capo):
  if hasattr(self.view, "set_capo"):
    self.view.set_capo(capo)
  else:
    self.capo = capo
    
song.Partition.set_capo = _set_capo

editor.register_attr("time"               , None)
editor.register_attr("mesure"             , None)
editor.register_attr("stringid"           , None)
editor.register_attr("linked_to"          , None)
editor.register_attr("linked_from"        , None)
editor.register_attr("stringid"           , None)
editor.register_attr("duration"           , DurationEditor)
editor.register_attr("value"              , NoteValueEditor)
editor.register_attr("text"               , None, song.Lyrics2)
editor.register_attr("pitch"              , editor.FloatEditor)
editor.register_attr("special_effect_name", editor.EnumEditor({
  song.Note.__name__        : _("Normal"),
  song.HammerNote.__name__  : _("Hammer / pull / legato"),
  song.BendNote.__name__    : _("Bend"),
  song.SlideNote.__name__   : _("Slide"),
  song.TremoloNote.__name__ : _("Tremolo"),
  song.DeadNote.__name__    : _("Dead note"),
  song.RollNote.__name__    : _("Roll"),
  }))

editor.register_attr("lines"              , None)
editor.register_attr("y"                  , None)
editor.register_attr("notationpos"        , editor.EnumEditor({ 0 : _("Upper"), 1 : _("Lower") }))
editor.register_attr("basenote"           , NoteValueEditor)
editor.register_attr("basenote"           , DrumPatchEditor, drum.String)

editor.register_attr("syncope"            , editor.BoolEditor)
editor.register_attr("tempo"              , editor.IntEditor)
editor.register_attr("rythm1"             , editor.IntEditor)
editor.register_attr("rythm2"             , editor.IntEditor)
editor.register_attr("duration"           , None, song.Mesure)

import staff
editor.register_attr("tonality"           , editor.EnumEditor(dict(map(lambda tonality: (tonality, _(tonality)), staff.TONALITIES.keys()))))

def _set_tonality(self, tonality):
  if hasattr(self.view, "set_tonality"):
    self.view.set_tonality(tonality)
  else:
    self.tonality = tonality
    
song.Partition.set_tonality = _set_tonality

editor.register_attr("MIDI_COMMAND"       , editor.StringEditor)
editor.register_attr("MIDI_USE_TEMP_FILE" , None)
editor.register_attr("PREVIEW_COMMAND"    , editor.StringEditor)
editor.register_attr("PRINT_COMMAND"      , None) # No longer used
editor.register_attr("PLAY_LOOP"          , editor.BoolEditor)
editor.register_attr("PREVIOUS_FILES"     , None)
editor.register_attr("NUMBER_OF_CANCEL"   , editor.RangeEditor(0, 100))

def edit(obj, **kargs): editobj.edit(obj, right_menu = 0, dialog = 1, **kargs)
